# coding=utf-8


cluster_start = object()
cluster_shutdown = object()

request_scheduled = object()
request_ignored = object()
response_received = object()

item_scraped = object()
item_ignored = object()
